#include <QMessageBox>
#include <QMouseEvent>
#include <QMetaProperty>
#include "mainwindow.h"
#include "ui_mainwindow.h"

MainWindow::MainWindow(QWidget *parent) :
    QMainWindow(parent),
    ui(new Ui::MainWindow)
{
    ui->setupUi(this);

    // Create loader.
    loader = new QUiLoader();

    // Fill in combo box with available widget types.
    ui->widgetComboBox->addItems(loader->availableWidgets());

    // Connect signals to slots.
    connect(ui->quitButton,  &QPushButton:: clicked, qApp, &QApplication::quit);
    connect(ui->createWidgetButton,  &QPushButton:: clicked, this, &MainWindow::CreateWidget);
    connect(ui->clearButton,  &QPushButton:: clicked, this, &MainWindow::Clear);
    connect(ui->infoButton,  &QPushButton:: clicked, this, &MainWindow::Info);
}

void MainWindow::CreateWidget()
{
    statusBar()->showMessage(tr("Creating %1.").arg(ui->widgetComboBox->currentText()), 1000);

    // Create widget as a child of the frame.
    QWidget *w = loader->createWidget(ui->widgetComboBox->currentText(), ui->frame, ui->widgetComboBox->currentText());

    // Show it.
    w->show();
}

void MainWindow::Clear()
{
    statusBar()->showMessage(tr("Deleting all widgets."), 1000);

    // Delete all child widgets of the frame.
    for (const auto &c : ui->frame->children()) {
        delete c;
    }
}

void MainWindow::Info()
{
    QString msg;

    // Iterate over all child widgets.
    for (const auto &c : ui->frame->children()) {
        msg += tr("Object name: %1<br>").arg(c->objectName());
        msg += tr("Class name: %1<br>").arg(c->metaObject()->className());

        const QMetaObject *metaObject = c->metaObject();
        QStringList properties;
        for (int i = metaObject->propertyOffset(); i < metaObject->propertyCount(); ++i)
            msg += tr("&nbsp;&nbsp;Property: %1<br>").arg(metaObject->property(i).name());
    }

    if (msg.isEmpty()) {
        msg = tr("No child widgets.");
    }

    // Show information in a dialog.
    QMessageBox::information(this, tr("Widget Information"), msg);
}

// Handle mouse event - get and save offset.
void MainWindow::mousePressEvent(QMouseEvent *event)
{
    offset = event->pos();
    statusBar()->showMessage(tr("Mouse press: %1, %2.").arg(event->pos().x()).arg(event->pos().y()), 1000);
}

// Handle mouse move event. Move widget under mouse if middle mouse button pressed.
void MainWindow::mouseMoveEvent(QMouseEvent *event)
{
    if (event->buttons() & Qt::MiddleButton) {
        QWidget *w = ui->frame->childAt(event->x(), event->y());
        if (w != nullptr) {
            statusBar()->showMessage(tr("Moving from %1, %2 to %3, %4.").arg(w->x()).arg(w->y()).arg(event->x()).arg(event->y()), 1000);
            w->move(event->pos() - offset);
        } else {
            statusBar()->showMessage(tr("No child widget."), 1000);
        }
    }
}

MainWindow::~MainWindow()
{
    delete ui;
}
